/* -----------------------------------------------------------------------------
 * Antenna - An Ant-to-end solution for wireless Java 
 *
 * Copyright (c) 2002-2004 Joerg Pleumann <joerg@pleumann.de>
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -----------------------------------------------------------------------------
 */
package de.pleumann.antenna;
import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.MatchingTask;

import de.pleumann.antenna.misc.Conditional;
import de.pleumann.antenna.misc.Utility;
import de.pleumann.antenna.pre.Preprocessor;

public class WtkPreprocess extends MatchingTask {

    private Conditional condition;

    private File sourceDir;

    private File targetDir;

    private String defines;

    private String newext;

    private String encoding;

    private int mode = Preprocessor.MODE_NORMAL | Preprocessor.MODE_INDENT;

    private Utility utility;

    public void init() {
        super.init();

        utility = Utility.getInstance(getProject(), this);

        condition = new Conditional(getProject());
        sourceDir = getProject().resolveFile(".");
        createInclude().setName("**/*.java");
    }

    public void setSrcdir(File value) {
        sourceDir = value;
    }

    public void setDestdir(File value) {
        targetDir = value;
    }

    public void setSymbols(String value) {
        defines = value;
    }

    public void setVerbose(boolean verbose) {
        if (verbose) {
            mode = mode | Preprocessor.MODE_VERBOSE;
        }
        else {
            mode = mode & ~Preprocessor.MODE_VERBOSE;
        }
    }

    public void setBackup(boolean backup) {
        if (backup) {
            mode = mode | Preprocessor.MODE_BACKUP;
        }
        else {
            mode = mode & ~Preprocessor.MODE_BACKUP;
        }
    }

    public void setIndent(boolean indent) {
        if (indent) {
            mode = mode | Preprocessor.MODE_INDENT;
        }
        else {
            mode = mode & ~Preprocessor.MODE_INDENT;
        }
    }

    public void setTest(boolean test) {
        if (test) {
            mode = mode | Preprocessor.MODE_TEST;
        }
        else {
            mode = mode & ~Preprocessor.MODE_TEST;
        }
    }

    public void setFilter(boolean filter) {
        if (filter) {
            mode = mode | Preprocessor.MODE_FILTER;
        }
        else {
            mode = mode & ~Preprocessor.MODE_FILTER;
        }
    }

    public void setNewext(String newext) {
        this.newext = newext;
    }

    public void setEncoding(String encoding) {
          this.encoding = encoding;
    }

    public void setIf(String s) {
        condition.setIf(s);
    }

    public void setUnless(String s) {
        condition.setUnless(s);
    }

    public boolean isActive() {
        return condition.isActive();
    }

    public void execute() throws BuildException {
        if (!isActive()) return;

        if (targetDir == null) {
            throw new BuildException("Need a target directory");
        }

        String[] files = getDirectoryScanner(sourceDir).getIncludedFiles();

        log("Preprocessing " + files.length + " file(s)");

        try {
            Preprocessor.execute(sourceDir, targetDir, files, defines, mode, newext, encoding, utility);
        }
        catch (Exception ex) {
            throw new BuildException("Preprocessing failed: " + ex.getMessage(), ex);
        }
    }
}
